--==========================================================================================================================
-- INCLUDES
--==========================================================================================================================
----------------------------------------------------------------------------------------------------------------------------
include("FLuaVector.lua")
print("The Chola Set Sail!")

local iUnitAdmiral = GameInfoTypes.UNIT_GREAT_ADMIRAL
local eCivilizationChola = GameInfoTypes.CIVILIZATION_VP_CHOLA
local eBuildingThirvai = GameInfoTypes.BUILDING_VP_THIRVAI
local eBuildingPekkandru = GameInfoTypes.BUILDING_D_FOR_PEKKANDRU_1
----------------------------------------------------------------------------------------------------------------------------
-- Thirisadai Birth
----------------------------------------------------------------------------------------------------------------------------
function ThirsadaiBirth(iPlayer, iUnit, iUnitType, iX, iY)
	--Abort if the new unit is not a Chola Great Admiral
	local pPlayer = Players[iPlayer]
	if not (pPlayer and pPlayer:GetCivilizationType() == eCivilizationChola) then return end

	if iUnitType == iUnitAdmiral then 
		local iUnitThirisadai = GameInfoTypes.UNIT_VP_THIRISADAI

		local pNewThirisadai = pPlayer:InitUnit(iUnitThirisadai, iX, iY, -1, DirectionTypes.NO_DIRECTION, false)
		pNewThirisadai:SetMoves(240)
		
		local pPlot = Map.GetPlot(iX, iY)
		local pCity = pPlot:GetPlotCity()
		local iFreeXP = pCity:GetDomainFreeExperience(pNewThirisadai:GetDomainType())
		pNewThirisadai:SetExperience(iFreeXP)
	end
end

function CholaTurnFunctions(iPlayer)
	local pPlayer = Players[iPlayer]
	
	if not (pPlayer and (pPlayer:GetCivilizationType() == eCivilizationChola)) then return end	
		
	if (pPlayer:IsAlive() and (not pPlayer:IsBarbarian())) then	
		-----------------------------------
		-- Kallarani Production, UA Gold, & Thirvai reset
		-----------------------------------
		local iUniqueTradePartnerCount = 0
		local iPekkandruCount = 0
		local eThirvaiTradeBonus = GameInfoTypes.BUILDING_THIRVAI_TRADE_BONUS
		
		for i, eachPlayer in pairs(Players) do
			if eachPlayer:GetCivilizationType() ~= iPlayer then

				iPekkandruCount = iPekkandruCount + eachPlayer:CountNumBuildings(eBuildingPekkandru)
				
				if eachPlayer:IsConnectedToPlayer(iPlayer) then
					iUniqueTradePartnerCount = iUniqueTradePartnerCount + 1
				end
			end
        end
		for city in pPlayer:Cities() do
			-- reset the TR count from UB, give the UA gold per city count, and adjust UU2 production
			city:SetNumRealBuilding(GameInfoTypes.BUILDING_D_FOR_PEKKANDRU_2, iPekkandruCount)
			city:SetNumRealBuilding(eThirvaiTradeBonus, 0)
			
			local iUnitKallarani = GameInfoTypes.UNIT_VP_KALLARANI
			if city:CanTrain(iUnitKallarani, 0, 1) then
				local iFreeKallaraniProduction = iUniqueTradePartnerCount * 5
				city:SetUnitProduction(iUnitKallarani, math.max(city:GetUnitProduction(iUnitKallarani), iFreeKallaraniProduction))
			end
		end
		-----------------------------------
		-- Thirvai Trade Bonus
		-----------------------------------
		local tRoutes = pPlayer:GetTradeRoutes()
		for i, route in ipairs(tRoutes) do
		
			local iOriginCity = route.FromCity
			if iOriginCity:IsHasBuilding(eBuildingThirvai) then

				if route.ToID ~= iPlayer then
					
					local iTargetCity = route.ToCity
					if iTargetCity:IsHasBuilding(eBuildingPekkandru) then 
					
						iOriginCity:SetNumRealBuilding(eThirvaiTradeBonus, iOriginCity:GetNumRealBuilding(eThirvaiTradeBonus) + 1)
					end
				end
			end
		end
		-----------------------------------
		-- Thirvai Vassal Bonus
		-----------------------------------
		local pTeam = Teams[pPlayer:GetTeam()]
		if pTeam:GetNumVassals() > 0 then
			print("Cholan Vassals")
			for city in pPlayer:Cities() do
				if city:IsHasBuilding(eBuildingThirvai) then
		
					local iThirvaiVassalBonus = math.floor(0.15 * 0.01 * pPlayer:GetMyShareOfVassalTaxes())
					city:SetNumRealBuilding(GameInfoTypes.BUILDING_THIRVAI_VASSAL_BONUS, iThirvaiVassalBonus)
					print("Thirvai Vassal Bonus")
				end
			end
		end
	end
end

function CholaTREndFrom(iFromPlayer, iFromCity, iToPlayer, iToCity, iDomain, iConnectionType)
	local pFromPlayer = Players[iFromPlayer]
	
	if not (pFromPlayer and (pFromPlayer:GetCivilizationType() == eCivilizationChola)) then return end	
	-----------------------------------
	-- UA Pekkandru Placement
	-----------------------------------
	if iFromPlayer ~= iToPlayer then
		local pToPlayer = Players[iToPlayer]
		local pToCity = pToPlayer:GetCityByID(iToCity)
		if not pToCity:IsHasBuilding(eBuildingPekkandru) then
			pToCity:SetNumRealBuilding(eBuildingPekkandru, 1)
		
			if pFromPlayer:IsHuman() and pFromPlayer:IsTurnActive() then
				local sName = pToCity:GetName()
				local iX, iY = pToCity:GetX(), pToCity:GetY()
				
				pFromPlayer:AddNotification(NotificationTypes.NOTIFICATION_INSTANT_YIELD,
					'The 500 Lords have established a Trading Guild in the City of '..sName..'. +1 [ICON_GOLD] Gold in all your Cities.',
					'Ainuruvar Trade Guild spreads to '..sName, 
					iX, iY, pToCity:GetID())
			end
		end
	end
	-----------------------------------
	-- Thirvai Admiral Bonus
	-----------------------------------
	local pFromCity = pFromPlayer:GetCityByID(iFromCity)
	if pFromCity:IsHasBuilding(eBuildingThirvai) then
		local iChange = 20 * (math.max(pFromPlayer:GetCurrentEra(), 1))
		pFromPlayer:ChangeNavalCombatExperience(iChange)
		
		if pFromPlayer:IsHuman() and pFromPlayer:IsTurnActive() then
			local vCityPosition1 = PositionCalculator(pFromCity:GetX(), pFromCity:GetY())
					
			Events.AddPopupTextEvent(vCityPosition1, "[COLOR_WHITE]+"..iChange.." [ICON_GREAT_ADMIRAL][ENDCOLOR]", 1)
		end
	end
end


function CholaTREndTo(iFromPlayer, iFromCity, iToPlayer, iToCity, iDomain, iConnectionType)
	local pToPlayer = Players[iToPlayer]
	
	if not (pToPlayer and (pToPlayer:GetCivilizationType() == eCivilizationChola)) then return end	
	
	local pToCity = pToPlayer:GetCityByID(iToCity)
	if pToCity:IsHasBuilding(eBuildingThirvai) then
		local iChange = 25 * (math.max(pToPlayer:GetCurrentEra(), 1))
		pToPlayer:ChangeNavalCombatExperience(iChange)
			
		if pToPlayer:IsHuman() and pToPlayer:IsTurnActive() then
			local vCityPosition2 = PositionCalculator(pToCity:GetX(), pToCity:GetY())
				
			Events.AddPopupTextEvent(vCityPosition2, "[COLOR_WHITE]+"..iChange.." [ICON_GREAT_ADMIRAL][ENDCOLOR]", 1)
		end
	end
end

function CholaReservoirPlace(iX, iY, iOwner, iOldImprovement, iNewImprovement, bPillaged)
	local pPlayer = Players[iOwner]

	if not pPlayer then return end
	
	local iKovilKulam = GameInfoTypes.IMPROVEMENT_CHOLA_KOVIL_KULAM
	local fReservoir = GameInfoTypes.FEATURE_VP_RESERVOIR
	local pMainPlot = Map.GetPlot(iX, iY)
	
	if iNewImprovement == iKovilKulam then			
		if (pMainPlot:GetOwner() ~= nil and Players[pMainPlot:GetOwner()]:GetCivilizationType() == eCivilizationChola) then
			pMainPlot:SetFeatureType(fReservoir)
		end
	end
	if iOldImprovement == iKovilKulam then
		pMainPlot:SetFeatureType(-1)
	end
end

function PositionCalculator(i1, i2)
	return HexToWorld(ToHexFromGrid(Vector2(i1, i2)))
end

if Game.IsCivEverActive(eCivilizationChola) then
	GameEvents.UnitCreated.Add(ThirsadaiBirth)
	GameEvents.PlayerDoTurn.Add(CholaTurnFunctions)
	GameEvents.PlayerTradeRouteCompleted.Add(CholaTREndFrom)
	GameEvents.PlayerTradeRouteCompleted.Add(CholaTREndTo)
	GameEvents.TileImprovementChanged.Add(CholaReservoirPlace)
end